import 'package:easy_ads/interstitial_ad/easy_interstital_ad.dart';
import 'package:firebase_admob/firebase_admob.dart';
import 'package:static_sharedpreferences/static_sharedpreferences.dart';

import '../easy_ads.dart';

class HealthyEasyInterstitialAd extends EasyInterstitialAd {
  final String _blockAdsUntilCountKey = EasyAds.SP_KEY +
      AdType.HEALTHY_INTERSTITIAL.toString() +
      "_blockAdsUntilCount";

  int get blockAdsUntilCount => _blockAdsUntilCount;
  int _blockAdsUntilCount;

  /// ## Description
  ///
  /// * Can only be changed in the constructor.
  ///
  ///
  /// * Amount of times a certain event has to
  ///   be executed until the `InterstitialAd` will be shown
  ///
  ///
  /// * This will be initialized with `50`
  int get blockAdsUntil => _blockAdsUntil;
  int _blockAdsUntil;

  int _count;

  /// ## Description
  ///
  /// * You `have to initialize` [EasyAds.initialize], otherwise this class will not work.
  ///
  ///
  /// * For more information regarding the parameters see [EasyInterstitialAd].
  ///
  ///
  /// * This class helps you not to flood your users with ads.
  ///
  ///
  /// ## Example
  ///
  /// {@tool sample}
  /// ```dart
  /// void main(){
  ///   WidgetsFlutterBinding.ensureInitialized();
  ///
  ///   EasyAds.initialize();
  ///
  ///   EasyAds.initEasyInterstitialAd(
  ///     HealthyEasyInterstitialAd(
  ///       blockAdsUntil: 5,
  ///     ),
  ///   );
  ///
  ///   runApp(MyApp());
  /// }
  ///
  /// class MyApp extends StatelessWidget {
  ///   @override
  ///   Widget build(BuildContext context) {
  ///     return MaterialApp(
  ///       title: 'EasyAds Demo',
  ///       theme: ThemeData(
  ///         primarySwatch: Colors.blue,
  ///       ),
  ///       home: MyAdWidget(),
  ///     );
  ///   }
  /// }
  ///
  /// class MyAdWidget extends StatefulWidget {
  ///   @override
  ///   _MyAdWidgetState createState() => _MyAdWidgetState();
  /// }
  ///
  /// class _MyAdWidgetState extends State<MyAdWidget> {
  ///
  ///   @override
  ///   Widget build(BuildContext context) {
  ///     return Scaffold(
  ///       body: Container(
  ///         child: Center(
  ///           child: RaisedButton(
  ///             // Because we set blockAdsUntil = 5 in the
  ///             // HealthyEasyInterstitialAd constructor
  ///             // the ad will not be shown until the
  ///             // user has clicked the button at least 5 times
  ///             onPressed: EasyAds.showEasyInterstitialAd,
  ///           ),
  ///         ),
  ///       ),
  ///     );
  ///   }
  /// }
  /// ```
  /// {@end-tool}
  HealthyEasyInterstitialAd({
    int blockAdsUntil = 50,
    String adUnitId,
    MobileAdTargetingInfo targetingInfo,
    AdLoadedFunction onAdLoaded,
    AdClickedFunction onAdClicked,
    AdOpenedFunction onAdOpened,
    AdImpressionFunction onAdImpression,
    AdClosedFunction onAdClosed,
    AdLeftApplicationFunction onLeftApplication,
    AdFailedToLoadFunction onAdFailedToLoad,
    MobileAdEventTriggeredFunction onAdEvent,
  }) : super(
          adUnitId: adUnitId,
          targetingInfo: targetingInfo,
          onAdLoaded: onAdLoaded,
          onAdClicked: onAdClicked,
          onAdOpened: onAdOpened,
          onAdImpression: onAdImpression,
          onAdClosed: onAdClosed,
          onLeftApplication: onLeftApplication,
          onAdFailedToLoad: onAdFailedToLoad,
          onAdEvent: onAdEvent,
        ) {
    _blockAdsUntil = blockAdsUntil;
    _count = 0;
    SP.execute(_restoreValues);
  }

  /// returns:
  /// ```
  /// AdType.HEALTHY_INTERSTITIAL
  /// ```
  @override
  AdType get adType => AdType.HEALTHY_INTERSTITIAL;

  void _saveValues() {
    int cache = SP.getInt(_blockAdsUntilCountKey) ?? 1;
    if (cache != _blockAdsUntilCount)
      SP.setInt(_blockAdsUntilCountKey, cache + _blockAdsUntilCount);
  }

  void _restoreValues() {
    _blockAdsUntilCount = SP.getInt(_blockAdsUntilCountKey) ?? 1;
  }

  /// Only shows the InterstitialAd if it has been loaded
  /// and blockAdsUntil has been reached.
  @override
  Future<bool> show({
    double anchorOffset = 0.0,
    double horizontalCenterOffset = 0.0,
    AnchorType anchorType = AnchorType.bottom,
  }) {
    if (_blockAdsUntil == _blockAdsUntilCount) {
      _saveValues();
      return super.show(
        anchorOffset: anchorOffset,
        horizontalCenterOffset: horizontalCenterOffset,
        anchorType: anchorType,
      );
    } else {
      _blockAdsUntilCount++;
      _saveValues();
      return null;
    }
  }

  /// ## Description
  ///
  /// * The Example assumes, that you have already initialized the
  ///   plugin with [EasyAds.initialize].
  ///
  ///
  /// * This is not affected by `blockAdsUntil`
  ///
  ///
  /// ## Parameters
  ///
  /// * `onValue`:
  ///
  ///   Assuming `onValue = 5`, then every fifth time this function gets
  ///   executed an ad will be shown.
  ///
  ///
  /// * `onShowAd`:
  ///
  ///   This function gets executed right before the [EasyInterstitialAd.show] function.
  ///
  ///
  /// * `onNotShowAd`:
  ///
  ///   In case the expression that evaluates if an ad will be shown,
  ///   using onValue and a private static counter variable equals false,
  ///   then this function gets executed.
  ///
  ///
  /// * `showAdOnValue`:
  ///
  ///   In case you do not want this function to automatically show an ad
  ///   if the expression that evaluates if an ad will be shown equals true,
  ///   then set this to `false`.
  ///
  ///
  /// ## Example
  ///
  /// {@tool sample}
  /// ```dart
  /// class MyAdWidget extends StatelessWidget {
  ///   @override
  ///   Widget build(BuildContext context) {
  ///     return Scaffold(
  ///       body: Container(
  ///         child: Center(
  ///           child: RaisedButton(
  ///             onPressed: () {
  ///               // Because we set onValue: 3,
  ///               // every third button press will show an ad,
  ///               // but before the ad is shown onShowAd
  ///               // will get executed if it is non-null.
  ///               //
  ///               // In case the user pressed the button for the first or
  ///               // second time, onNotShowAd will get executed
  ///               // if non-null.
  ///               EasyAds.easyInterstitialAd.showOnCount(
  ///                 onValue: 3,
  ///                 onShowAd: () {
  ///                   print("now showing ad");
  ///                 },
  ///                 onNotShowAd: () {
  ///                   print("u r lucky, no ad");
  ///                 },
  ///               );
  ///             },
  ///           ),
  ///         ),
  ///       ),
  ///     );
  ///   }
  /// }
  /// ```
  /// {@end-tool}
  void showOnCount({
    int onValue = 5,
    Function onShowAd,
    Function onNotShowAd,
    bool showAdOnValue = true,
  }) {
    if (_count == onValue) {
      _count = 0;
      if (onShowAd != null) onShowAd();
      if (showAdOnValue) super.show();
    } else {
      _count++;
      if (onNotShowAd != null) onNotShowAd();
    }
  }

  /// ## Description
  ///
  /// * This is affected by `blockAdsUntil`,
  ///   everything else is the same as [showOnCount].
  void showOnCountAffected({
    int onValue = 5,
    Function onShowAd,
    Function onNotShowAd,
    bool showAdOnValue = true,
  }) {
    if (_count == onValue) {
      _count = 0;
      if (onShowAd != null) onShowAd();
      if (showAdOnValue) show();
    } else {
      _count++;
      if (onNotShowAd != null) onNotShowAd();
    }
  }
}
